/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *             fixed checkBlockIn2                     *
 *                                                     *
 *******************************************************/

#include "ComputerPlayer.h"

ComputerPlayer::ComputerPlayer():Player("Computer")
{
     initWinIn2Matrix();
}

/* The computer decides where to place its piece based on the following
   algorithm:
             1 - if the computer has 2 in a row then it will take the win
             2 - if the player has 2 in a row then it will block the win
             3 - if the player can win in 2 moves then it will block the win
*/
void ComputerPlayer::playAMove(GameBoard& currentBoard)
{
     int row = -1, col = -1;
     bool pieceHasBeenPlaced = false;

     // get the current game board so it can be used to determine where the
     // computer will move.
     currentBoard.getBoard(board);
     
     // Check if comp can win in one move     
     if (checkOneMove(row, col, get_piece()))
     {
         if (get_piece() == get_X_VAL())
             currentBoard.x_move(row, col);
         else
             currentBoard.o_move(row, col);
             
         pieceHasBeenPlaced = true;
     }
     
     // Check if comp needs to block a win
     if (pieceHasBeenPlaced == false && checkOneMove(row, col, 
                  ((get_piece() == get_X_VAL())? get_O_VAL():get_X_VAL())) )
     {
         if (get_piece() == get_X_VAL())
             currentBoard.x_move(row, col);
         else
             currentBoard.o_move(row, col);
         
         pieceHasBeenPlaced = true;
     }

     // Place piece to block win in 2 moves     
     if (pieceHasBeenPlaced == false && checkBlockIn2(row, col))
     {
         if (get_piece() == get_X_VAL())
             currentBoard.x_move(row, col);
         else
             currentBoard.o_move(row, col);
         
         pieceHasBeenPlaced = true;
     }
     
     // Place piece on next available spot
     if (pieceHasBeenPlaced == false && moveToNextAvailableSpace(row, col))
     {
         if (get_piece() == get_X_VAL())
             currentBoard.x_move(row, col);
         else
             currentBoard.o_move(row, col);
     }
}

bool ComputerPlayer::checkOneMove(int& row, int& col, int boardVal)
{
     // check one move win is possible
     
     bool hasTwoInRow = false;
     int numInRow = 0;
     int locOfMove = -1;
     
     // check rows
     for (int i = 0; i < 3 && hasTwoInRow == false; i++)
     {
         numInRow = 0;
         locOfMove = -1;
         for (int j = 0; j < 3; j++)
         {
             if (board[i][j] == boardVal)
                 numInRow++;
             else if (board[i][j] == get_BLANK_VAL())
                 locOfMove = j;
         }
         if (numInRow == 2 && locOfMove != -1)
         {
             hasTwoInRow = true;
             row = i;
             col = locOfMove;
         }
     }
     
     // check cols
     for (int i = 0; i < 3 && hasTwoInRow == false; i++)
     {
         numInRow = 0;
         locOfMove = -1;
         for (int j = 0; j < 3; j++)
         {
             if (board[j][i] == boardVal)
                 numInRow++;
             else if (board[j][i] == get_BLANK_VAL())
                 locOfMove = j;
         }
         if (numInRow == 2 && locOfMove != -1)
         {
             hasTwoInRow = true;
             col = i;
             row = locOfMove;
         }
     }
     
     numInRow = 0;
     locOfMove = -1;
     
     // check diags
     if (hasTwoInRow == false)
     {
         if (board[0][0] == boardVal)
             numInRow++;
         else if (board[0][0] == get_BLANK_VAL())
             locOfMove = 0;
             
         if (board[1][1] == boardVal)
             numInRow++;
         else if (board[1][1] == get_BLANK_VAL())
             locOfMove = 1;
         
         if (board[2][2] == boardVal)
             numInRow++;
         else if (board[2][2] == get_BLANK_VAL())
             locOfMove = 2;
         
         if (numInRow == 2 && locOfMove != -1)
         {
             row = locOfMove;
             col = locOfMove;
             hasTwoInRow = true;
         }
     }
     
     numInRow = 0;
     locOfMove = -1;
     
     // Check off diag
     if (hasTwoInRow == false)
     {
         if (board[0][2] == boardVal)
             numInRow++;
         else if (board[0][2] == get_BLANK_VAL())
             locOfMove = 2;
             
         if (board[1][1] == boardVal)
             numInRow++;
         else if (board[1][1] == get_BLANK_VAL())
             locOfMove = 1;
         
         if (board[2][0] == boardVal)
             numInRow++;
         else if (board[2][0] == get_BLANK_VAL())
             locOfMove = 20;
           
         if (numInRow == 2 && locOfMove != -1)
         {
             if (locOfMove == 2)
             {
                 row = 0;
                 col = 2;
             }
             else if (locOfMove == 1)
             {
                 row = 1;
                 col = 1;
             }
             else if (locOfMove == 20)
             {
                 row = 2;
                 col = 0;
             }
             hasTwoInRow = true;
             
         }
     }
     
     return hasTwoInRow;
}

bool ComputerPlayer::moveToNextAvailableSpace(int& row, int& col)
{
     // find the next available blank space to move to
     // Priority is given to the middle position, if that is not available then
     // priority is high at top left and low at bottom right
     
     bool blankSpaceFound = false;
     
     if (board[1][1] == get_BLANK_VAL())
     {
         blankSpaceFound = true;
         row = 1;
         col = 1;
     }
     
     for (int r = 0; r < 3 && blankSpaceFound == false; r++)
     {
         for (int c = 0; c < 3 && blankSpaceFound == false; c++)
         {
             if (board[r][c]==get_BLANK_VAL())
             {
                 blankSpaceFound = true;
                 row = r;
                 col = c;
             }
         }
     }
     
     return blankSpaceFound;
}

void ComputerPlayer::initWinIn2Matrix()
{
     // There are 6 conditions in which you can win in two moves. These board
     // setups are initialized here. These 6 conditions can also occur in any
     // of the 4 90 deg rotations.
     
     // NOTE: -99 is a don't care position. That is we don't care if there is
     // a piece there or if it is blank.
     
     winIn2[0][0][0] = get_BLANK_VAL();
     winIn2[0][1][0] = get_BLANK_VAL();
     winIn2[0][2][0] = get_X_VAL();
     winIn2[1][0][0] = get_BLANK_VAL();
     winIn2[1][1][0] = get_O_VAL();
     winIn2[1][2][0] = get_BLANK_VAL();
     winIn2[2][0][0] = get_X_VAL();
     winIn2[2][1][0] = get_BLANK_VAL();
     winIn2[2][2][0] = get_BLANK_VAL();
     
     locForMove[0][0] = 0;
     locForMove[0][1] = 1;
     
     winIn2[0][0][1] = get_BLANK_VAL();
     winIn2[0][1][1] = get_X_VAL();
     winIn2[0][2][1] = get_BLANK_VAL();
     winIn2[1][0][1] = get_X_VAL();
     winIn2[1][1][1] = get_O_VAL();
     winIn2[1][2][1] = -99;
     winIn2[2][0][1] = get_BLANK_VAL();
     winIn2[2][1][1] = -99;
     winIn2[2][2][1] = -99;
     
     locForMove[1][0] = 0;
     locForMove[1][1] = 0;
     
     winIn2[0][0][2] = get_BLANK_VAL();
     winIn2[0][1][2] = get_BLANK_VAL();
     winIn2[0][2][2] = get_X_VAL();
     winIn2[1][0][2] = get_X_VAL();
     winIn2[1][1][2] = get_O_VAL();
     winIn2[1][2][2] = -99;
     winIn2[2][0][2] = get_BLANK_VAL();
     winIn2[2][1][2] = -99;
     winIn2[2][2][2] = -99;
     
     locForMove[2][0] = 0;
     locForMove[2][1] = 0;
     
     winIn2[0][0][3] = get_BLANK_VAL();
     winIn2[0][1][3] = get_BLANK_VAL();
     winIn2[0][2][3] = get_X_VAL();
     winIn2[1][0][3] = get_BLANK_VAL();
     winIn2[1][1][3] = get_O_VAL();
     winIn2[1][2][3] = -99;
     winIn2[2][0][3] = get_X_VAL();
     winIn2[2][1][3] = -99;
     winIn2[2][2][3] = -99;
     
     locForMove[3][0] = 0;
     locForMove[3][1] = 0;
     
     winIn2[0][0][5] = -99;
     winIn2[0][1][5] = get_BLANK_VAL();
     winIn2[0][2][5] = -99;
     winIn2[1][0][5] = -99;
     winIn2[1][1][5] = get_X_VAL();
     winIn2[1][2][5] = -99;
     winIn2[2][0][5] = -99;
     winIn2[2][1][5] = get_BLANK_VAL();
     winIn2[2][2][5] = -99;
     
     locForMove[5][0] = 0;
     locForMove[5][1] = 1;
     
     winIn2[0][0][4] = get_BLANK_VAL();
     winIn2[0][1][4] = -99;
     winIn2[0][2][4] = -99;
     winIn2[1][0][4] = -99;
     winIn2[1][1][4] = get_X_VAL();
     winIn2[1][2][4] = -99;
     winIn2[2][0][4] = -99;
     winIn2[2][1][4] = -99;
     winIn2[2][2][4] = get_BLANK_VAL();
     
     locForMove[4][0] = 0;
     locForMove[4][1] = 0;
}

void ComputerPlayer::rotateBoard()
{
     // This function will rotate the current game board that is saved within
     // the computerPlayer class.
     
     // rotate and save it in tempBoard
     int tempBoard[3][3];
     int iRotated = 0;
     int jRotated = 0;
     
     for (int i = -1; i < 2; i++)
     {
         for (int j = -1; j < 2; j++)
         {
             iRotated = -j;
             jRotated = i;
             tempBoard[-1*(jRotated-1)][iRotated+1] = board[-1*(j-1)][i+1];
         }
     }
     
     // save tempBoard into board
     for (int i = 0; i < 3; i++)
     {
         for (int j = 0; j < 3; j++)
         {
             board[i][j] = tempBoard[i][j];
         }
     }
}

bool ComputerPlayer::checkBlockIn2(int& row, int& col)
{
     bool compareResult = false;
     int tempRow = 0;
     int tempCol = 0;
     int temp = 0;
     
     for (int i = 0; i < 6; i++)
     {
         // check all 6 win in two moves configuration
         
         tempCol = locForMove[i][1]-1;
         tempRow = -1*locForMove[i][0]+1;
         for (int j = 0; j < 4; j++)
         {
             // if the board matches one of the preset configurations
             // block the win.
             compareResult = compare(i);
             if (compareResult == true)
             {
                 row = -1*(tempRow-1);
                 col = tempCol+1;
                 return true;
             }
             // rotate the board by 90 and check the same configuration
             rotateBoard();
             temp = tempRow;
             tempRow = -tempCol;
             tempCol = temp;
         }
     }
     
     return compareResult;
}

bool ComputerPlayer::compare(int boardPossition)
{
     // Check if the game bord matches one of the previously defined
     // win in 2 move configurations.
     if (board[locForMove[boardPossition][0]][locForMove[boardPossition][1]] 
         != get_BLANK_VAL())
         return false;
     
     bool compareResult = true;
         
     for (int i = 0; i < 3; i++)
     {
         for (int j = 0; j < 3; j++)
         {
             if (winIn2[i][j][boardPossition] != -99)
             {
                if (board[i][j] != winIn2[i][j][boardPossition])
                   compareResult = false;
             }
         }
     }
     
     return compareResult;
}
