/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *             added definitions for                   *
 *                  UI_H_VS_C                          *
 *                  UI_H_VS_H                          *
 *                  UI_AGAIN_SAME_PLAYERS              *
 *                  UI_AGAIN_NEW_PLAYERS               *
 *                  UI_QUIT                            *
 *                                                     *
 *******************************************************/
 
 /*
 This is a TicTacToe game class. It is capable of playing
 a Human vs Human or Human vs Computer game of TicTacToe.
 
 This class requires the use of the GameBoard, Player,
 ComputerPlayer and HumanPlayer classes.
 */

#ifndef GAME_H
#define GAME_H

#include <string>
using namespace std;

#include "HumanPlayer.h"
#include "ComputerPlayer.h"
#include "GameBoard.h"

// Insted of using constants we can also define a variable. The defined variable
// is not created in memory as is the case with constants, insted when the code
// is compiled the defined string is replaced with its definition. That is when
// compiling this code UI_H_VS_C is replaced with 1. If I had created UI_H_VS_C
// as a constant then a memory space will be created for it.
#define UI_H_VS_C 1  // Human verses computer
#define UI_H_VS_H 2  // Human verses human

#define UI_AGAIN_SAME_PLAYERS 1
#define UI_AGAIN_NEW_PLAYERS 2
#define UI_QUIT 3
    
// status of game
//   G_INPGROGRESS - 0 - game not over
//   G_TIE         - 1 - game end, tie game
//   G_P1_WON      - 2 - game end, Player 1 won
//   G_P2_WON      - 3 - game end, Player 2 won
//   G_QUIT        - 4 - game end because player quit
enum TTTGameStatus {G_INPROGRESS, G_TIE, G_P1_WON, G_P2_WON, G_QUIT};

class Game
{
public:

    // Constructor
    Game();
    
    ~Game();
    
    // Ask the user for player info.
    void setPlayers();
    
    // Reset the game including stats and player info
    void resetGameAndPlayer();
    
    // Play one game.
    // OUTPUT: 1  - successfully played a game
    //         -1 - unsuccessfull
    // SIDE EFECT: This will set the game results to player objects
    int playAGame();
    
    // Will display the current player stats.
    void printPlayerStats();
    
    // Ask the user if they want to play again.
    // OUTPUT: UI_AGAIN_SAME_PLAYERS - user want to continue playing
    //         UI_AGAIN_NEW_PLAYERS  - user want to continue playing but with
    //                                 new players
    //         UI_QUIT               - user wants to quit
    int playAgain();
    
    // Reset just the board and game status
    void resetGame();
    
private:

    // The game board used for this game
    GameBoard board;
    
    // This game's two players
    // Since we don't know if the game will have computer players or human
    // players, we will declare the two variables as pointers to the parent 
    // player class. When creating the object we will decide if it will be the
    // HumanPlayer or ComputerPlayer siblings.
    Player* Player1;
    Player* Player2;
    
    // current status of game
    TTTGameStatus GameStatus;
    
    // Number of games played before reseting
    int numGames;
    
    // Indicates which player will go next
    int playersMove;
    
    // Flag to check if this is a player vs computer game. If this is false it is
    // assumed that it is a player vs player game.
    // NOTE: you should name bool variables in the form of a yes or no question.
    //       This will make it easy for people to understand your code.
    bool isPlayerVsComputer;
    
    // Ask the user for what type of game it is.
    // OOUTPUT: UI_H_VS_C - Human vs Computer
    //          UI_H_VS_H - Human vs Human
    // NOTE: We will not have a computer vs computer game.
    int getTypeOfGame();
    
    // Ask the user for player name and what type of playing piece to use
    // INPUT: Player object for seting name and piece.
    //        getPieceFromUser - if true ask user for what playing piece to use
    //                           else don't ask user
    void getPlayerProperties(Player* CurrentPlayer, bool getPieceFromUser);
    
    // If game is over display the result of the game.
    void endGame();
    
    // Determine the current status of the game. Should be called after each
    // player move.
    void setGameStatus();
};

#endif
