/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *                                                     *
 *******************************************************/
 
  /*
 This is a TicTacToe board class. It keeps track of the
 pieces as well as the status of the board ie if someone
 has 3 in a row or not.
 */
 
#ifndef GAMEBOARD_H
#define GAMEBOARD_H

#include <iostream>
#include <stdlib.h>
#include <string>
using namespace std;

// Game bord status
// INPROGRESS - 0 - no winner, game can still continue
// X_WIN      - 1 - X win
// O_WIN      - 2 - O win
// TIE        - 3 - Tie
// QUIT       - 4 - Quit game
enum GameBoardStatus {INPROGRESS, X_WIN, O_WIN, TIE, QUIT};

class GameBoard
{
public:

    // Constructer resets game board and gamestatus.
    GameBoard();
    
    // This function will display the current board status.
    void display_board()const;
    
    // This function will put an X at the row and col on the game board.
    // INPUTS: row,col is the coordinate where the X will be put
    // OUTPUT: 1 - move made sucessfully
    //        -1 - invalid move
    int x_move(int row, int col);
    
    // This function will put an O at the row and col on the game board.
    // INPUTS: row,col is the coordinate where the O will be put
    // OUTPUT: 1 - move make sucessfully
    //        -1 - invalid move
    int o_move(int row, int col);
    
    // Returns the game status
    // OUTPUT: Game bord status
    //           X_WIN  - X win
    //           O_WIN  - O win
    //           TIE    - Tie
    //           QUIT   - Quit game
    GameBoardStatus getBoardStatus() const
    {
        return BoardStatus;
    };
    
    // This function will put the member matrix i_GameBoard to the iput
    // argument.
    // INPUT: board - the argument to which the i_GameBoard will be set to it
    void getBoard(int board[][3]) const;
    
    // This function will reset the board to Blank
    void resetBoard();
    
    // Early quit of the game
    void quitGame();

private:

    // The game board
    int i_GameBoard[3][3];
    
    // The current status of the board
    GameBoardStatus BoardStatus;
    
    // The following are values for the game board.
    // This could also have been created using #define
    static const int X_VAL = 1;
    static const int O_VAL = 2;
    static const int BLANK_VAL = -1;
    
    // This function will check the board status
    // EFFECT: Sets the i_BoardStatus variable
    void boardStatus();
    
    // This function will check if the player val has 3 TicTacToe
    // The function will assume that the correct val is sent in.
    // INPUT: val is the value of X or O
    // OUTPUT: true for TicTacToe
    //         false for no TicTacToe
    bool isThreeInRow(int val) const;
    
    // This function will check if the game is a tie.
    // The game is a tie only if there is no place for a player to move to.
    // OUTPUT: true for a tie
    bool isTie() const;
};

#endif
